/*==============================================================================


       eg_3 - file I/O example


==============================================================================*/
#include <string.h>

#include "gui.h"
#include "ebm_object.h"

extern "C"
{
#include "ereader_hostio.h"
}
/*==============================================================================

       CPaFile - file I/O object

==============================================================================*/
#ifndef BYTE
typedef unsigned char BYTE;
#endif
//------------------------------------------------------------------------------
class CPaFile
{

  private:

    ebo_name_t m_fileInfo;
    BYTE  *m_ptr;
    size_t m_size;
    size_t m_maxSize;
    bool   m_open;
    bool   m_readWrite;
    bool   m_newFile;

  public:

    CPaFile(const ebo_name_t*info);
    ~CPaFile(void);
    void Close(void);
    bool Resize(unsigned int newSize);
    bool Open(bool readAndWrite);
    bool ReadOnly(void)     // readonly status of file
    {  return ! m_readWrite; }
    bool IsOpen(void)       // checks if file currently open (mapped)
    {  return m_open;        }
    size_t Size(void)       // returns size of open file
    {  return m_size;        }
    BYTE *Ptr(void)         // returns pointer to memory mapping - invalid if file not open!
    {  return m_ptr;         }
    bool BrandNewFile(void) // set after each Open() - returns whether the file was just created then or not
    {  return m_newFile;     }


};
//------------------------------------------------------------------------------
CPaFile::CPaFile(const ebo_name_t*info)
 : m_ptr(0), m_size(0), m_maxSize(0),
   m_open(false), m_readWrite(false), m_newFile(false)
{
  memcpy(&m_fileInfo,info,sizeof(ebo_name_t)); // keep our own copy
  m_maxSize=ebo_roundup(8000000UL);            // set aside more than enough space
  m_ptr=(BYTE*)ebo_roundup(OS_availaddr);     // align on 4k boundary beyond last memory address used
  OS_availaddr=(U32)m_ptr+m_maxSize;                // protect our memory by adjusting the available memory pointer
}
//------------------------------------------------------------------------------
CPaFile::~CPaFile(void)
{
  Close(); // close if still open
}
//------------------------------------------------------------------------------
bool CPaFile::Open(bool readAndWrite)
{
  if (m_open)
    Close(); // we can't work on open (mapped) files!
  m_newFile=false;
  size_t size=m_maxSize;
  m_readWrite=readAndWrite;
  if (0> ebo_mapin(&m_fileInfo,0,m_ptr,&size,readAndWrite))
  {
    if ( 0>ebo_new(&m_fileInfo,EBO_BLK_SIZE,1)) // failed opening - try to create new small file
      return false; // failed to new - catastrophic error
    m_newFile=true; // let everyone know we just created this file
    if ( 0>ebo_mapin(&m_fileInfo,0,m_ptr,&size,readAndWrite) ) // map in this new file
      return false; // failed again!
  }
  // at this point, we have a valid file mapped in
  m_size=size;
  m_open=true;
  return true;
}
//------------------------------------------------------------------------------
void CPaFile::Close(void)
{
  if (m_open)
  {
    ebo_unmap(m_ptr,m_maxSize);
    m_open=false;
  }
}
//------------------------------------------------------------------------------
bool CPaFile::Resize(unsigned int newSize)
{
  newSize= ebo_roundup (newSize); // make sure even 4k blocks
  if (newSize==m_size) // if we don't need to expand or contract, quit now
    return true;
  bool oldFlag=m_open, readWrite=m_readWrite; // save for after
  if (m_open)
    Close(); // can't work on mapped file
  bool failure=true;
  if (newSize<m_size)
  {
    if (0<=ebo_contract(&m_fileInfo,newSize)) // success?
    {
      m_size=newSize;
      failure=false;
    }
  }
  else
  {
    if (0<=ebo_extend(&m_fileInfo,newSize)) // success?
    {
      m_size=newSize;
      failure=false;
    }
  }
  if (oldFlag) // open again if was previously opened, and no failure
    failure = failure || ( !Open(readWrite) ); // keep flag after opening
  return !failure;
}
/*==============================================================================

       CWindow1/eg_3 - test window

==============================================================================*/
class CWindow1 : public CWindow
{

  private:

    enum
    {
      ID_CWindow1=100,
      ID_Checkbox1,
      ID_Rect1,
      ID_Label1, ID_Label2, ID_Label3, ID_Label4, ID_Label5, ID_Label6,
      ID_Caption, ID_Button1, ID_Button2, ID_Button3, ID_Button4, ID_Button5, ID_Button6,
    };

    CCheckbox *Checkbox1;
    CTextEdit *Caption;

    int  m_count;
    CPaFile *m_file;

    S32  MsgHandler( MSG_TYPE type, CViewable *from, S32 data );
    void DisplayFileStatus(void);

  public:

    CWindow1(void);
    ~CWindow1(void);

};
//------------------------------------------------------------------------------
static const ebo_name_t c_progFile = { "","Pankhurst Algorithmics","TEST","TXT" };
//------------------------------------------------------------------------------
CWindow1::CWindow1(void)
 : CWindow( ID_CWindow1,0,0,200,240,"Example 3 - File I/O",FALSE,FALSE,TRUE)
{
  AddChild(new CLabel(ID_Label1,"Write a string", 0),92, 84);
  AddChild(new CLabel(ID_Label2,"Read a string",  0),92,112);
  AddChild(new CLabel(ID_Label3,"Expand the file",0),92,140);
  AddChild(new CLabel(ID_Label4,"Shrink the file",0),92,168);
  AddChild(new CLabel(ID_Label5,"Close the file", 0),92,196);
  AddChild(new CRect(ID_Rect1,188,2),4,44);
  AddChild(new CButton(ID_Button1,72,20,"Open"),    8, 52);
  AddChild(new CButton(ID_Button2,72,20,"Write"),   8, 80);
  AddChild(new CButton(ID_Button3,72,20,"Read"),    8,108);
  AddChild(new CButton(ID_Button4,72,20,"Expand"),  8,136);
  AddChild(new CButton(ID_Button5,72,20,"Contract"),8,164);
  AddChild(new CButton(ID_Button6,72,20,"Close"),   8,192);
  Checkbox1=new CCheckbox(ID_Checkbox1,92,13," Read Only");
  AddChild(Checkbox1,88,56);
  Caption=new CTextEdit(ID_Caption,180,36,TEXTOPTION_NOEDIT|TEXTOPTION_NOUNDERLINE);
  AddChild(Caption,8,4);
  // set up file for later access
  m_file=new CPaFile(&c_progFile);
  m_count=0;
  DisplayFileStatus();
}
//------------------------------------------------------------------------------
CWindow1::~CWindow1(void)
{
  delete m_file; // closes if necessary
}
//------------------------------------------------------------------------------
S32 CWindow1::MsgHandler( MSG_TYPE type, CViewable *from, S32 data )
{
  if ( MSG_BUTTON_SELECT==type )
  {
    switch (from->GetID() )
    {
      case ID_Button1: // open
      {
        bool readOnly=Checkbox1->GetDownStatus();
        if (m_file->Open( ! readOnly ))
          GUI_Alert(ALERT_WARNING,"Opened");
        else
          GUI_Alert(ALERT_WARNING,"File NOT opened!");
      }
        break;
      case ID_Button2: // write - we just write a tring to start of file
      {
        char *ptr=(char*)m_file->Ptr(),  buff[99]="";
        sprintf(buff,"Example string #%d",++m_count);        
        strcpy(ptr,buff); // will fail if file not open!
        GUI_Alert(ALERT_WARNING,"String written");
      }
        break;
      case ID_Button3: // read a string from start of file
      {
        char *ptr=(char*)m_file->Ptr(),  buff[99]="String read is: ";
        strcat(buff,ptr); // will fail if file not open!
        GUI_Alert(ALERT_WARNING,buff);
      }
        break;
      case ID_Button4: // expand
      {
        int size=m_file->Size()+EBO_BLK_SIZE; // make sure we add a block to it
        if (m_file->Resize(size))
          GUI_Alert(ALERT_WARNING,"File enlarged");
        else
          GUI_Alert(ALERT_WARNING,"File NOT enlarged!");
      }
        break;
      case ID_Button5: // contract
      {
        int size=m_file->Size()-EBO_BLK_SIZE; // can go negative!
        if (m_file->Resize(size))
          GUI_Alert(ALERT_WARNING,"File shrunk");
        else
          GUI_Alert(ALERT_WARNING,"File NOT shrunk!");
      }
        break;
      case ID_Button6: // close
        m_file->Close();
        GUI_Alert(ALERT_WARNING,"File closed");
        break;
    }
    DisplayFileStatus();
    return 1;
  }
  return CWindow::MsgHandler(type, from, data);
}
//------------------------------------------------------------------------------
void CWindow1::DisplayFileStatus(void)
{
  char buff[299]="Status: TEXT.TXT ";
  if ( ! m_file->IsOpen() )
  {
    strcat(buff,"not open");
  }
  else
  {
    strcat(buff,"Opened ");
    if (m_file->ReadOnly())
      strcat(buff,"read-only, ");
    else
      strcat(buff,"read/write, ");
    sprintf(&buff[strlen(buff)],"%d bytes long",m_file->Size());
    if (m_file->BrandNewFile())
      strcat(buff,", file just created on Open()");
  }
  Caption->SetText(buff);
}
/*==============================================================================


==============================================================================*/
static CWindow1 *f_mainWindow;
//------------------------------------------------------------------------------
S32 GUI_main( MSG_TYPE type, CViewable *object, S32 data )
{
  switch (type)
  {
    case MSG_APP_START:
      if (OS_is_present && hostIO_am_I_the_current_task())
        HOSTIO_INLINE_BREAKPOINT();
      f_mainWindow = new CWindow1();
      return 1;
    case MSG_APP_STOP: // prog exit & cleanup
      delete f_mainWindow;
      return 1;
    default:
      break;
  }
  return 0;
};
//------------------------------------------------------------------------------



